package org.faceless.pdf;

import java.util.WeakHashMap;
import java.lang.ref.*;

abstract class PeeredObject
{
    private static final WeakHashMap map = new WeakHashMap();

    PeeredObject() {}

    abstract Object getPeer();

    static final Object getPeer(Object newobject)
    {
	if (newobject==null) return null;
        Object oldobject = map.get(newobject);
        if (oldobject!=null) {
            return ((Reference)oldobject).get();
        } else {
	    if (newobject instanceof org.faceless.pdf2.PDFPage) {
	        oldobject=new PDFPage((org.faceless.pdf2.PDFPage)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.PDFAction) {
	        oldobject=new PDFAction((org.faceless.pdf2.PDFAction)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.PDFAnnotation) {
	        oldobject=new PDFAnnotation((org.faceless.pdf2.PDFAnnotation)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.PDFFont) {
	        oldobject=new PDFFont((org.faceless.pdf2.PDFFont)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.PDFStyle) {
	        oldobject=new PDFStyle((org.faceless.pdf2.PDFStyle)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.PDFImage) {
	        oldobject=new PDFImage((org.faceless.pdf2.PDFImage)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.PDFPattern) {
	        oldobject=new ColorPattern((org.faceless.pdf2.PDFPattern)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.PDFBookmark) {
	        oldobject=new PDFBookmark((org.faceless.pdf2.PDFBookmark)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.FormButton) {
	        oldobject=new FormButton((org.faceless.pdf2.FormButton)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.FormText) {
	        oldobject=new FormText((org.faceless.pdf2.FormText)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.FormRadioButton) {
	        oldobject=new FormRadioButton((org.faceless.pdf2.FormRadioButton)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.FormCheckbox) {
	        oldobject=new FormCheckbox((org.faceless.pdf2.FormCheckbox)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.FormSignature) {
	        oldobject=new FormSignature((org.faceless.pdf2.FormSignature)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.FormChoice) {
	        oldobject=new FormChoice((org.faceless.pdf2.FormChoice)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.Form) {
	        oldobject=new Form((org.faceless.pdf2.Form)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.PDFSound) {
	        oldobject=new PDFSound((org.faceless.pdf2.PDFSound)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.LayoutBox.Text) {
	        oldobject=new LayoutBox.Text((org.faceless.pdf2.LayoutBox.Text)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.LayoutBox.Box) {
	        oldobject=new LayoutBox.Box((org.faceless.pdf2.LayoutBox.Box)newobject);
	    } else if (newobject instanceof org.faceless.pdf2.LayoutBox) {
	        oldobject=new LayoutBox((org.faceless.pdf2.LayoutBox)newobject);
	    } else {
	        throw new Error("Unknown peer class "+newobject.getClass());
	    }
	    map.put(newobject, new WeakReference(oldobject));
	}
	return oldobject;
    }

    public String toString()
    {
        return getPeer().toString();
    }
}
