// $Id: ZoomLevel.java 15793 2012-06-11 13:30:29Z mike $

package org.faceless.pdf2.viewer2.feature;

import org.faceless.pdf2.viewer2.*;
import org.faceless.pdf2.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;

/**
 * Create a widget which displays the current zoom level, and allows the user to
 *
 * edit it to set the zoom level.
 * <span class="featurename">The name of this feature is <span class="featureactualname">ZoomLevel</span></span>
 * <p><i>This code is copyright the Big Faceless Organization. You're welcome to use, modify and distribute it in any form in your own projects, provided those projects continue to make use of the Big Faceless PDF library.</i></p>
 * @since 2.8
 */
public class ZoomLevel extends ViewerWidget implements DocumentPanelListener {

    private JComboBox field;
    private boolean fromupdate;

    public ZoomLevel() {
        super("ZoomLevel");

        field = new JComboBox(new String[] { "15%", "25%", "50%", "75%", "100%", "125%", "150%", "200%", "400%", "800%", "1600%", "2400%", "3200%", "6400%"});
        field.setSelectedItem("100%");
        field.putClientProperty("JComponent.sizeVariant", "small");         // Improve OS X/Nimbus appearance
        field.setEditable(true);
        field.setFont(null);
        field.setEnabled(false);
        field.setPrototypeDisplayValue("6400%");    // Why does this give such bad results?
        if (field.getEditor().getEditorComponent() instanceof JTextField) {
            int cols = Util.isLAFAqua() ? 3 : 4;
            ((JTextField)field.getEditor().getEditorComponent()).setColumns(cols);
        }
        field.setToolTipText(UIManager.getString("PDFViewer.tt.ZoomLevel"));
        field.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                DocumentPanel docpanel = getViewer().getActiveDocumentPanel();
                if (docpanel != null && field.getSelectedItem() != null) {
                    float oldzoom = docpanel.getZoom();
                    String s = (String)field.getSelectedItem();
                    try {
                        if (s.endsWith("%")) {
                            s = s.substring(0, s.length() - 1);
                        }
                        float newzoom = Float.parseFloat(s)/100f;
                        if (Math.abs(newzoom-oldzoom) > 0.01) {
                            docpanel.getViewport().setZoomMode(DocumentViewport.ZOOM_NONE);
                            docpanel.setZoom(newzoom);
                            if (!fromupdate) {  // This is a result of a click! Turn off ZoomFitN
                                ToggleViewerWidget w = (ToggleViewerWidget)getViewer().getFeature(ZoomFit.class);
                                if (w != null && w.isSelected()) {
                                    w.setSelected(false);
                                }
                                w = (ToggleViewerWidget)getViewer().getFeature(ZoomFitWidth.class);
                                if (w != null && w.isSelected()) {
                                    w.setSelected(false);
                                }
                                w = (ToggleViewerWidget)getViewer().getFeature(ZoomFitHeight.class);
                                if (w != null && w.isSelected()) {
                                    w.setSelected(false);
                                }

                                docpanel.getViewport().setZoomMode(DocumentViewport.ZOOM_NONE);
                            }
                            fromupdate = false;
                        }
                    } catch (Exception e) {
                        field.setSelectedItem(Integer.toString((int)Math.round(oldzoom*100))+"%");
                    }

                    Component focusowner = KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
                    if (focusowner != null && (focusowner == field || field.isAncestorOf(focusowner))) {
                        docpanel.getViewport().requestFocusInWindow();
                    }
                }
            }
        });
        setComponent("Navigation", field);
    }

    public void initialize(PDFViewer viewer) {
        super.initialize(viewer);
        viewer.addDocumentPanelListener(this);
    }

    public void documentUpdated(DocumentPanelEvent event) {
        if (event.getType()=="redrawn" || event.getType()=="activated") {
            fromupdate = true;          // We're called from a DocumentPanel being updated....
            field.setSelectedItem(Integer.toString((int)Math.round(event.getDocumentPanel().getZoom()*100))+"%");
        }
    }

}
